#coding:utf-8

import requests
import urllib.parse

class Winny(object):

    def __init__(self, host, port="", protocol="http"):
        self.host = host
        self.port = port
        self.protocol = protocol
        self.domain = "{}://{}".format(protocol, host)
        if port and port != 80:
            self.domain = self.domain+":"+str(port)
        self.RESULT_FORMATS = ["json", "unicode", "bytes"]
        self.result = None
    
    def _bind_func_url(self, url, method):
        def req(params=None, format="json"):
            print("url: ", url)
            return self.request(method, url, params)
        return req
    
    def add_url(self, method, uri, function_name):
        method = method.upper()
        function_name = function_name.lower()
        url = urllib.parse.urljoin(self.domain, uri)
        setattr(self, function_name, self._bind_func_url(url, method))
    
    def request(self, method, url, params=None):
        if method.upper() == "GET":
            return self.get(url, params)
        if method.upper() == "POST":
            return self.post(url, params)

    def get(self, url, params=None):
        assert url
        assert (not params or isinstance(params, dict))
        self.result = requests.get(url, params=params)
    
    def post(self, url, data=None):
        assert url
        assert (not data or isinstance(data, dict))
        self.result = requests.post(url, data=data)
    
    def put(self, url, data=None):
        assert url
        assert (not data or isinstance(data, dict))
        self.result = requests.put(url, data)
    
    def get_json(self):
        assert self.result
        if self.result.ok:
            return self.result.json
        raise Exception("Error, http status code = {}".format(self.result.status_code))
    
    def get_unicode(self):
        assert self.result
        if self.result.ok:
            return self.result.text
        raise Exception("Error, http status code = {}".format(self.result.status_code))
    
    def get_bytes(self):
        assert self.result
        if self.result.ok:
            return self.result.content
        raise Exception("Error, http status code = {}".format(self.result.status_code))


if __name__ == "__main__":
    wy = Winny(host="www.baidu.com")
    wy.add_url(method="get", uri="/", function_name="download")
    wy.download()
    t = wy.get_bytes()
    print(t)



#coding:utf-8

from os import path
from codecs import open
from setuptools import setup

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name = 'winney',
    version = '0.1.0',
    author = 'olivetree123',
    license = 'BSD',
    packages = ['winney'],
    author_email = 'olivetree123@163.com',
    url = 'https://github.com/olivetree123/Winny',
    description = 'Object-Oriented HTTP Request',
    long_description = long_description,
    long_description_content_type = 'text/markdown',
    install_requires = [
        "requests>=2.18.0",
    ],
)